<?php
namespace JHodges\GiteaBotPHPUnit;

use \JHodges\GiteaBot\Client;

/* creates a new issue for each failed test
* if it already exists then report is added as a comment
* if it passes then the issue is closed
*/

final class PostFailedTests{

    private $fails=[];
    private $client=null;
    private $repo=null;

    function __construct($xml_path){
        $this->xml = simplexml_load_file($xml_path);
    }

    public function post($url, $user, $pass, $repoUser, $repo){
        if(!$url) return;

        // open connection and repo
        $this->client=new Client($url, $user, $pass, true);
        $this->repo=$this->client->getRepo($repoUser, $repo);

        $this->process($this->xml);
    }


    private function fail($name,$att,$error,$message){
        //composer the issue
        $issueTitle="Failed test $name";
        $issueBody="";
        // upload any screenshots and add to message
        if(file_exists("/tmp/$name.png")){
            $data=$this->repo->addReleaseAttachment(1,"/tmp/$name.png");
            $url=$data->browser_download_url;
            $issueBody.="\n![]($url)\n";
            unlink("/tmp/$name.png");
        }
        //add the message
        if(trim($message)) $issueBody.="```plain\n$message.\n```\n";
        // add the atts
        $issueBody.=" * result = $error\n";
        foreach($att as $k=>$v){
            $issueBody.=" * $k = $v\n";
        }

        // does the issue exist
        $issue=$this->repo->getIssues(['q' => $issueTitle])[0]??null;
        if($issue){
            $issue->addComment($issueBody);
        }else{
            $issue=$this->repo->createIssue([
                'title'=>$issueTitle,
                'body'=>$issueBody
            ]);
        }
    }

    private function success($name,$att){
        $issue=$this->repo->getIssues(['q' => $name])[0]??null;

        if($issue){
            $issue->state='closed';
            $issue->save();
        }
    }

    private function process($item){
        if(isset($item->testcase)){
            foreach($item->testcase as $testcase) {
                $att=$testcase->attributes();
                $name=$testcase->attributes()->class[0].'::'.$testcase->attributes()->name[0];
                $fail=false;
                foreach($testcase as $k=>$v){
                    $this->fail($name,$att,$k,$v);
                    $fail=true;
                }
                if(!$fail){
                    $this->success($name,$att);
                }
            }
        }
        if(isset($item->testsuite)){
            foreach($item->testsuite as $testsuite) {
                $this->process($testsuite);
            }
        }
    }

}
